==> warcar-1.1.5-sources.jar: com/bxm/warcar/utils/file/SystemFileMonitor.java <==
package com.bxm.warcar.utils.file;

import com.bxm.warcar.utils.NamedThreadFactory;
import com.google.common.base.Preconditions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * @author allen
 * @since 1.0.0
 */
public class SystemFileMonitor implements FileMonitor {

    private static final Logger LOGGER = LoggerFactory.getLogger(SystemFileMonitor.class);
    private final FileComparator fileComparator;
    private final ScheduledExecutorService scheduled = new ScheduledThreadPoolExecutor(1, new NamedThreadFactory("file-monitor"));
    private UpdateListener listener;

    public SystemFileMonitor(FileComparator fileComparator) {
        Preconditions.checkNotNull(fileComparator);
        this.fileComparator = fileComparator;
    }

    @Override
    public void start() {
        try {
            // First initial
            fileComparator.compare();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // scheduled
        scheduled.scheduleWithFixedDelay(() -> {
            try {
                boolean compare = fileComparator.compare();
                if (!compare) {
                    if (null != listener) {
                        listener.updated();
                    }
                }
            } catch (IOException e) {
                if (LOGGER.isErrorEnabled()) {
                    LOGGER.error("compare: ", e);
                }
            }
        }, 0, 1, TimeUnit.MINUTES);
    }

    @Override
    public void addListener(UpdateListener listener) {
        this.listener = listener;
    }

    public static void main(String[] args) {
        File data = new File(System.getProperty("java.io.tmpdir") + File.separator + "ipipnet.data");
        File fingerprint = new File(System.getProperty("java.io.tmpdir") + File.separator + "ipipnet.fingerprint");
        FileComparator comparator = new SystemFileComparator(fingerprint, data,
                new SimpleFingerprintTracker("http://bxm-guide.oss-cn-shanghai.aliyuncs.com/dist/IP_File/ipipnet.fingerprint"),
                new SimpleDataTracker("http://bxm-guide.oss-cn-shanghai.aliyuncs.com/dist/IP_File/ipipnet.datx"));

        SystemFileMonitor monitor = new SystemFileMonitor(comparator);
        monitor.addListener(() -> {
            System.out.println("updated");
        });
        monitor.start();
    }
}

==> warcar-1.1.7.3-20181214.023322-2-sources.jar: com/bxm/warcar/utils/file/SystemFileMonitor.java <==
package com.bxm.warcar.utils.file;

import com.bxm.warcar.utils.NamedThreadFactory;
import com.google.common.base.Preconditions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * @author allen
 * @since 1.0.0
 */
public class SystemFileMonitor implements FileMonitor {

    private static final Logger LOGGER = LoggerFactory.getLogger(SystemFileMonitor.class);
    private final FileComparator fileComparator;
    private final ScheduledExecutorService scheduled = new ScheduledThreadPoolExecutor(1, new NamedThreadFactory("file-monitor"));
    private UpdateListener listener;

    public SystemFileMonitor(FileComparator fileComparator) {
        Preconditions.checkNotNull(fileComparator);
        this.fileComparator = fileComparator;
    }

    @Override
    public void start() {
        try {
            // First initial
            fileComparator.compare();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // scheduled
        scheduled.scheduleWithFixedDelay(() -> {
            try {
                boolean compare = fileComparator.compare();
                if (!compare) {
                    if (null != listener) {
                        listener.updated();
                    }
                }
            } catch (IOException e) {
                if (LOGGER.isErrorEnabled()) {
                    LOGGER.error("compare: ", e);
                }
            }
        }, 0, 1, TimeUnit.MINUTES);
    }

    @Override
    public void addListener(UpdateListener listener) {
        this.listener = listener;
    }

    public static void main(String[] args) {
        File data = new File(System.getProperty("java.io.tmpdir") + File.separator + "ipipnet.data");
        File fingerprint = new File(System.getProperty("java.io.tmpdir") + File.separator + "ipipnet.fingerprint");
        FileComparator comparator = new SystemFileComparator(fingerprint, data,
                new SimpleFingerprintTracker("http://bxm-guide.oss-cn-shanghai.aliyuncs.com/dist/IP_File/ipipnet.fingerprint", "ipipnet.fingerprint"),
                new SimpleDataTracker("http://bxm-guide.oss-cn-shanghai.aliyuncs.com/dist/IP_File/ipipnet.datx", "ipipnet.datx"));

        SystemFileMonitor monitor = new SystemFileMonitor(comparator);
        monitor.addListener(() -> {
            System.out.println("updated");
        });
        monitor.start();
    }
}
